![My Logo](https://www.naterscreations.com/imputegap/logo_imputegab.png)


# ImputeGAP - Datasets
ImputeGap uses several datasets containing different characteristics to test your implementations.



## EEG-Test

The **EEG-Test** dataset, owned by Henri Begleiter [EEG dataset](https://kdd.ics.uci.edu/databases/eeg/eeg.data.html), is utilized in various studies such as [Statistical mechanics of neocortical interactions: Canonical momenta indicatorsof electroencephalography](https://link.aps.org/doi/10.1103/PhysRevE.55.4578).
It describes an EEG database composed of individuals with a genetic predisposition to alcoholism.
The dataset contains measurements from 64 electrodes placed on subject's scalps which were sampled at 256 Hz (3.9-msec epoch) for 1 second.
The dataset contains a total of 416 samples.
The specific subset used in ImputeGAP is the S2 match for trial 119, identified as `co3a0000458.rd`.
The dataset's dimensions are 64 series, each containing 256 values.
This dataset is primarily used for the analysis of medical and brain-related data, with a focus on detecting predictable patterns in brain wave activity.

### Plots
The plots present a series of plots derived from the EEG-TEST dataset, illustrating various aspects of the data and preprocessing steps.
EEG-Test dataset - raw data 64x256 shows the full raw dataset, consisting of NxM time series.
EEG-Test dataset - raw data 20x400 provides a subset of the data, limited to 20 time series over 400 time steps, while EEG-Test dataset - raw data 01x400 focuses on a single time series extracted from the dataset.
Finally, EEG-Test - normalized 20x400 demonstrates the impact of "MIN-MAX" normalization on the raw data, applied to the same 20x400 subset.

![EEG-Test dataset - raw data 64x256](https://github.com/eXascaleInfolab/ImputeGAP/raw/main/imputegap/dataset/docs/eeg-test/01_eeg-test-rawdata-NxM_graph.jpg)
![EEG-Test dataset - raw data 20x400](https://github.com/eXascaleInfolab/ImputeGAP/raw/main/imputegap/dataset/docs/eeg-test/02_eeg-test-rawdata20x400_graph.jpg)
![EEG-Test dataset - raw data 01x400](https://github.com/eXascaleInfolab/ImputeGAP/raw/main/imputegap/dataset/docs/eeg-test/03_eeg-test-rawdata01x400_graph.jpg)
![EEG-Test - normalized 20x400](https://github.com/eXascaleInfolab/ImputeGAP/raw/main/imputegap/dataset/docs/eeg-test/04_eeg-test-normmin_maxdata01x400_graph.jpg)


### Features

| Category       | Feature                                                            | Value                |
|----------------|--------------------------------------------------------------------|----------------------|
| Geometry       | 5-bin histogram mode                                               | 1.0100240549492727   |
| Geometry       | 10-bin histogram mode                                              | 0.48777445966067257  |
| Geometry       | Proportion of high incremental changes in the series               | 0.9014222059451871   |
| Geometry       | Longest stretch of above-mean values                               | 272.0                |
| Geometry       | Transition matrix column variance                                  | 0.002397955611566198 |
| Geometry       | Goodness of exponential fit to embedding distance distribution     | 0.2767301848754665   |
| Geometry       | Positive outlier timing                                            | 0.004302978515625    |
| Geometry       | Negative outlier timing                                            | -0.015869140625      |
| Geometry       | Longest stretch of decreasing values                               | 13.0                 |
| Geometry       | Rescaled range fluctuation analysis (low-scale scaling)            | 0.48                 |
| Geometry       | Detrended fluctuation analysis (low-scale scaling)                 | 0.34                 |
| Correlation    | First 1/e crossing of the ACF                                      | 40.477031355783346   |
| Correlation    | First minimum of the ACF                                           | 5                    |
| Correlation    | Histogram-based automutual information (lag 2, 5 bins)             | 0.25175574018411473  |
| Correlation    | Time reversibility                                                 | 0.014249236968944982 |
| Correlation    | First minimum of the AMI function                                  | 4.0                  |
| Correlation    | Change in autocorrelation timescale after incremental differencing | 0.025423728813559324 |
| Trend          | Wangs periodicity metric                                           | 8                    |
| Trend          | Entropy of successive pairs in symbolized series                   | 1.633147167561509    |
| Trend          | Error of 3-point rolling mean forecast                             | 0.5702585744085695   |
| Transformation | Power in the lowest 20% of frequencies                             | 0.876724240984972    |
| Transformation | Centroid frequency                                                 | 0.02684466378800049  |




### Summary

| Data info          | Values                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  |
|--------------------|---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Dataset name       | eeg-test                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                |
| Dataset codename   | co3a0000458.rd                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| Dataset name       | EEG Database: Genetic Predisposition to Alcoholism                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      |
| Url                | https://kdd.ics.uci.edu/databases/eeg/eeg.data.html                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     |
 | Specific URL       | http://kdd.ics.uci.edu/databases/eeg/eeg_full.tar                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |                                            |
| Source             | UCI KDD Archive<br/>Henri Begleiter<br/>Neurodynamics Laboratory<br/>State University of New York Health Center<br/>Brooklyn, New York                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  | 
| Articles           | L. Ingber. (1997). Statistical mechanics of neocortical interactions: Canonical momenta indicators of electroencephalography. Physical Review E. Volume 55. Number 4. Pages 4578-4593.<br/><br/>L. Ingber. (1998). Statistical mechanics of neocortical interactions: Training and testing canonical momenta indicators of EEG. Mathematical Computer Modelling. Volume 27. Number 3. Pages 33-64.<br/><br/>J. G. Snodgrss and M. Vanderwart. (1980). "A standardized set of 260 pictures: norms for the naming agreement, familiarity, and visual complexity." Journal of Experimental Psychology: Human Learning and Memory. Volume 6. Pages 174-215. |
| Time granularity   | 1 second per measurement (3.9 ms epoch)                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 |
| Trials             | 120 trials                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              |
| Channels           | 64 channels                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             |
| Samples            | 416 samples (368 post-stim samples)                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     |
| Time resolution    | 3.906 ms uV                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             |
| Specific trial     | S2 match, trial 119                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     |                                                                                                                                                                                                                                       |
| Dataset dimensions | N=256 M=64 electrodes                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   |

<br /><hr /><br />


## EEG

The **EEG** dataset, created by the DERCo, is a collection of EEG recordings obtained from participants engaged in text reading tasks [A Dataset for Human Behaviour in Reading Comprehension Using {EEG}](https://www.nature.com/articles/s41597-024-03915-8).
This corpus includes behavioral data from 500 participants, as well as EEG recordings from 22 healthy adult native English speakers.
The dataset features a time resolution of 1000 Hz, with time-locked recordings from -200 ms to 1000 ms relative to the stimulus onset.
The dataset consists of 564 epochs, although only one was selected for this specific EEG subset.
The extracted dataset contains 1201 values across 33 series.
The goal of this dataset is to provide an alternative perspective on neuroscience-related datasets and to enable comparisons of results from different research studies utilizing the same technology.

### Plots
The plots present a series of plots derived from the EEG dataset, illustrating various aspects of the data and preprocessing steps.
EEG dataset - raw data 64x256 shows the full raw dataset, consisting of NxM time series.
EEG dataset - raw data 20x400 provides a subset of the data, limited to 20 time series over 400 time steps, while EEG dataset - raw data 01x400 focuses on a single time series extracted from the dataset.
Finally, EEG - normalized 20x400 demonstrates the impact of "MIN-MAX" normalization on the raw data, applied to the same 20x400 subset.

![EEG dataset - raw data 64x256](https://github.com/eXascaleInfolab/ImputeGAP/raw/main/imputegap/dataset/docs/eeg/01_eeg-rawdata-NxM_graph.jpg)
![EEG dataset - raw data 20x400](https://github.com/eXascaleInfolab/ImputeGAP/raw/main/imputegap/dataset/docs/eeg/02_eeg-rawdata20x400_graph.jpg)
![EEG dataset - raw data 01x400](https://github.com/eXascaleInfolab/ImputeGAP/raw/main/imputegap/dataset/docs/eeg/03_eeg-rawdata01x400_graph.jpg)
![EEG - normalized 20x400](https://github.com/eXascaleInfolab/ImputeGAP/raw/main/imputegap/dataset/docs/eeg/04_eeg-normmin_maxdata01x400_graph.jpg)


### Features


| Category       | Feature                                                           | Value                 |
|----------------|-------------------------------------------------------------------|-----------------------|
| Geometry       | 5-bin histogram mode                                              | -0.9695404504750353   |
| Geometry       | 10-bin histogram mode                                             | 0.8830459332128981    |
| Geometry       | Proportion of high incremental changes in the series              | 0.03237110802974915   |
| Geometry       | Longest stretch of above-mean values                              | 918.0                 |
| Geometry       | Transition matrix column variance                                 | 0.010398644752018455  |
| Geometry       | Goodness of exponential fit to embedding distance distribution    | 0.254572789901192     |
| Geometry       | Positive outlier timing                                           | 0.9746760449755458    |
| Geometry       | Negative outlier timing                                           | 0.9568144002420209    |
| Geometry       | Longest stretch of decreasing values                              | 60.0                  |
| Geometry       | Rescaled range fluctuation analysis (low-scale scaling)           | 0.18                  |
| Geometry       | Detrended fluctuation analysis (low-scale scaling)                | 0.12                  |
| Correlation    | First 1/e crossing of the ACF                                     | 9.848740939179107     |
| Correlation    | First minimum of the ACF                                          | 15                    |
| Correlation    | Histogram-based automutual information (lag 2, 5 bins)            | 0.06581564166675209   |
| Correlation    | Time reversibility                                                | 0.0027164281133996667 |
| Correlation    | First minimum of the AMI function                                 | 14.0                  |
| Correlation    | Change in autocorrelation timescale after incremental differencing | 0.02413793103448276   |
| Trend          | Wangs periodicity metric                                          | 27                    |
| Trend          | Entropy of successive pairs in symbolized series                  | 1.2800853946906863    |
| Trend          | Error of 3-point rolling mean forecast                            | 0.28083302897988865   |
| Transformation | Power in the lowest 20% of frequencies                            | 0.999395333262392     |
| Transformation | Centroid frequency                                                | 0.07526093240564423   |


### Summary

| Data info          |                                                                         |
|--------------------|-------------------------------------------------------------------------|
| Dataset codename   | eeg                                                                     |
| Dataset name       | DERCo: A Dataset for Human Behaviour in Reading Comprehension Using EEG |
| Url                | https://doi.org/10.17605/OSF.IO/RKQBU                                   |
 | Specific URL       | https://osf.io/tu4zj                                                    |
| Source             | DERCo: A Dataset for Human Behaviour in Reading Comprehension Using EEG |
| Article            | https://www.nature.com/articles/s41597-024-03915-8<br/>Boi Mai Quach, Cathal Gurrin & Graham Healy                      |
| Time granularity   | 1000.0 Hz                                                               |
| t                  | -200.00 ...    1000.00 ms                                                               |
| Epoch              | 1 used on 564                                                           |
| Dataset dimensions | N=1201 M=33                                                             |

<br /><hr /><br />


## Drift
The Drift dataset comprises 13,910 measurements collected from 16 chemical sensors exposed to six different gases, with only batch 10 utilized for this dataset [Gas Sensor Array Drift at Different Concentrations](https://archive.ics.uci.edu/dataset/270).
It includes information on the concentration levels to which the sensors were exposed during each measurement.
Data was collected over a 36-month period, from January 2008 to February 2011, at a gas delivery platform facility within the ChemoSignals Laboratory at the BioCircuits Institute, University of California, San Diego [On the calibration of sensor arrays for pattern recognition using the minimal number of experiments](https://www.sciencedirect.com/science/article/pii/S0169743913001937).
The dataset has a time granularity of 6 hours and consists of 100 time series, each containing 1,000 data points. This dataset is particularly valuable for testing algorithms designed to handle drift and outliers.

### Plots
The plots present a series of plots derived from the Drift dataset, illustrating various aspects of the data and preprocessing steps.
Drift dataset - raw data 64x256 shows the full raw dataset, consisting of NxM time series.
Drift dataset - raw data 20x400 provides a subset of the data, limited to 20 time series over 400 time steps, while Drift dataset - raw data 01x400 focuses on a single time series extracted from the dataset.
Finally, Drift - normalized 20x400 demonstrates the impact of "MIN-MAX" normalization on the raw data, applied to the same 20x400 subset.

![Drift dataset - raw data 64x256](https://github.com/eXascaleInfolab/ImputeGAP/raw/main/imputegap/dataset/docs/drift/01_drift-rawdata-NxM_graph.jpg)
![Drift dataset - raw data 20x400](https://github.com/eXascaleInfolab/ImputeGAP/raw/main/imputegap/dataset/docs/drift/02_drift-rawdata20x400_graph.jpg)
![Drift dataset - raw data 01x400](https://github.com/eXascaleInfolab/ImputeGAP/raw/main/imputegap/dataset/docs/drift/03_drift-rawdata01x400_graph.jpg)
![Drift - normalized 20x400](https://github.com/eXascaleInfolab/ImputeGAP/raw/main/imputegap/dataset/docs/drift/04_drift-normmin_maxdata01x400_graph.jpg)

### Features


| Category       | Feature                                                            | Value                 |
|----------------|--------------------------------------------------------------------|-----------------------|
| Geometry       | 5-bin histogram mode                                               | -1.2468173605295707   |
| Geometry       | 10-bin histogram mode                                              | 0.8254787327262303    |
| Geometry       | Proportion of high incremental changes in the series               | 0.965619656196562     |
| Geometry       | Longest stretch of above-mean values                               | 92.0                  |
| Geometry       | Transition matrix column variance                                  | 0.0009604922097481461 |
| Geometry       | Goodness of exponential fit to embedding distance distribution     | 0.03029724105804939   |
| Geometry       | Positive outlier timing                                            | 0.006040000000000045  |
| Geometry       | Negative outlier timing                                            | 0.008399999999999963  |
| Geometry       | Longest stretch of decreasing values                               | 7.0                   |
| Geometry       | Rescaled range fluctuation analysis (low-scale scaling)            | 0.64                  |
| Geometry       | Detrended fluctuation analysis (low-scale scaling)                 | 0.84                  |
| Correlation    | First 1/e crossing of the ACF                                      | 0.5531405331698586    |
| Correlation    | First minimum of the ACF                                           | 1                     |
| Correlation    | Histogram-based automutual information (lag 2, 5 bins)             | 0.0030914312919998453 |
| Correlation    | Time reversibility                                                 | -0.007087613186347406 |
| Correlation    | First minimum of the AMI function                                  | 4.0                   |
| Correlation    | Change in autocorrelation timescale after incremental differencing | 1.0                   |
| Trend          | Wangs periodicity metric                                           | 3                     |
| Trend          | Entropy of successive pairs in symbolized series                   | 2.188715240115428     |
| Trend          | Error of 3-point rolling mean forecast                             | 1.1957075487620428    |
| Transformation | Power in the lowest 20% of frequencies                             | 0.17090456872484874   |
| Transformation | Centroid frequency                                                 | 1.8841598396202843    |

### Summary

| Data info              |                                                                                                                                                                                       |
|------------------------|---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Dataset codename       | drift                                                                                                                                                                                 |
| Dataset names          | Gas Sensor Array Drift Dataset at Different Concentrations                                                                                                                            |
| Source                 | Alexander Vergara (vergara '@' ucsd.edu)<br/>BioCircutis Institute<br/>University of California San DiegoSan Diego, California, USA                                                   |                                                                                                                                    |
| Donors of the Dataset: | Alexander Vergara (vergara '@' ucsd.edu)<br/>Jordi Fonollosa (fonollosa '@'ucsd.edu)<br/>Irene Rodriguez-Lujan (irrodriguezlujan '@' ucsd.edu)<br/>Ramon Huerta (rhuerta '@' ucsd.edu) |                                                                                                                                                                                    |
| Time granularity       | 6 hours                                                                                                                                                                               |
| Dataset dimensions     | N=1000 M=100                                                                                                                                                                          |
| Remarks                | only batch 10 is taken from the dataset                                                                                                                                               |
| Url                    | https://archive.ics.uci.edu/ml/datasets/Gas+Sensor+Array+Drift+Dataset+at+Different+Concentrations                                                                                    |


<br /><hr /><br />


## Chlorine

The Chlorine dataset originates from chlorine residual management aimed at ensuring the security of water distribution systems [Chlorine Residual Management for Water Distribution System Security](https://www.researchgate.net/publication/226930242_Chlorine_Residual_Management_for_Water_Distribution_System_Security), with data sourced from [US EPA Research](https://www.epa.gov/research).
It consists of 50 time series, each representing a distinct location, with 1,000 data points per series recorded at 5-minute intervals.
The dataset exhibits a cyclic pattern with recurring peaks, suggesting the presence of periodic characteristics.
This makes it particularly well-suited for time series imputation methods that are designed to detect and leverage seasonality in data.

### Plots
The plots present a series of plots derived from the Chlorine dataset, illustrating various aspects of the data and preprocessing steps.
Chlorine dataset - raw data 64x256 shows the full raw dataset, consisting of NxM time series.
Chlorine dataset - raw data 20x400 provides a subset of the data, limited to 20 time series over 400 time steps, while Chlorine dataset - raw data 01x400 focuses on a single time series extracted from the dataset.
Finally, Chlorine - normalized 20x400 demonstrates the impact of "MIN-MAX" normalization on the raw data, applied to the same 20x400 subset.

![Chlorine dataset - raw data 64x256](https://github.com/eXascaleInfolab/ImputeGAP/raw/main/imputegap/dataset/docs/chlorine/01_chlorine-rawdata-NxM_graph.jpg)
![Chlorine dataset - raw data 20x400](https://github.com/eXascaleInfolab/ImputeGAP/raw/main/imputegap/dataset/docs/chlorine/02_chlorine-rawdata20x400_graph.jpg)
![Chlorine dataset - raw data 01x400](https://github.com/eXascaleInfolab/ImputeGAP/raw/main/imputegap/dataset/docs/chlorine/03_chlorine-rawdata01x400_graph.jpg)
![Chlorine - normalized 20x400](https://github.com/eXascaleInfolab/ImputeGAP/raw/main/imputegap/dataset/docs/chlorine/04_chlorine-normmin_maxdata01x400_graph.jpg)

### Features

| Category       | Feature                                                            | Value                 |
|----------------|--------------------------------------------------------------------|-----------------------|
| Geometry       | 5-bin histogram mode                                               | -0.7816940400450461   |
| Geometry       | 10-bin histogram mode                                              | -0.988389310201491    |
| Geometry       | Proportion of high incremental changes in the series               | 0.123702474049481     |
| Geometry       | Longest stretch of above-mean values                               | 970.0                 |
| Geometry       | Transition matrix column variance                                  | 0.06770833333333334   |
| Geometry       | Goodness of exponential fit to embedding distance distribution     | 0.24298075807561495   |
| Geometry       | Positive outlier timing                                            | -0.86082              |
| Geometry       | Negative outlier timing                                            | 0.3675600000000001    |
| Geometry       | Longest stretch of decreasing values                               | 124.0                 |
| Geometry       | Rescaled range fluctuation analysis (low-scale scaling)            | 0.24                  |
| Geometry       | Detrended fluctuation analysis (low-scale scaling)                 | 0.32                  |
| Correlation    | First 1/e crossing of the ACF                                      | 1559.7693638923522    |
| Correlation    | First minimum of the ACF                                           | 62                    |
| Correlation    | Histogram-based automutual information (lag 2, 5 bins)             | 1.150419457226549     |
| Correlation    | Time reversibility                                                 | -0.004961660957249964 |
| Correlation    | First minimum of the AMI function                                  | 40.0                  |
| Correlation    | Change in autocorrelation timescale after incremental differencing | 0.004814305364511692  |
| Trend          | Wangs periodicity metric                                           | 122                   |
| Trend          | Entropy of successive pairs in symbolized series                   | 1.186724208570797     |
| Trend          | Error of 3-point rolling mean forecast                             | 0.08211532856608073   |
| Transformation | Power in the lowest 20% of frequencies                             | 0.9988700796218077    |
| Transformation | Centroid frequency                                                 | 0.0004793689962142944 |


### Summary

| Data info          |                                                                                                                                                                                    |
|--------------------|------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Dataset codename   | chlorine                                                                                                                                                                           |
| Dataset name       | Chlorine data                                                                                                                                                                      |
| Url                | https://www.epa.gov/research                                                                                                                                                       |
| Source             | United States Environmental Protection Agency, EPANET<br/>Prof. Jeanne M. VanBriesen                                                                                               |
| Article            | Vanbriesen, Jeanne & Parks, Shannon & Helbling, Damian & Mccoy, Stacia. (2011). Chlorine Residual Management for Water Distribution System Security. 10.1007/978-1-4614-0189-6_11. | 
| Time granularity   | 5 minutes                                                                                                                                                                          |
| Dataset dimensions | N=1000 M=50                                                                                                                                                                        |

<br /><hr /><br />

## Climate

The Climate dataset is an aggregated and processed collection used for climate change attribution studies.
It contains observations data for 18 climate agents across 125 locations in North America [USC Melady Lab](https://viterbi-web.usc.edu/~liu32/data.html).
The dataset has a temporal granularity of 1 month, comprising 10 series with 5,000 values each.
This structure is particularly valuable for spatio-temporal modeling [Spatial-temporal causal modeling for climate change attribution](https://dl.acm.org/doi/10.1145/1557019.1557086), as it enables researchers to account for both spatial and temporal dependencies.
The dataset exhibits high variability, along with periodic or trend-like behavior and noise, making it suitable for advanced analytical techniques.

### Plots
The plots present a series of plots derived from the Climate dataset, illustrating various aspects of the data and preprocessing steps.
Climate dataset - raw data 64x256 shows the full raw dataset, consisting of NxM time series.
Climate dataset - raw data 20x400 provides a subset of the data, limited to 20 time series over 400 time steps, while Climate dataset - raw data 01x400 focuses on a single time series extracted from the dataset.
Finally, Climate - normalized 20x400 demonstrates the impact of "MIN-MAX" normalization on the raw data, applied to the same 20x400 subset.

![Climate dataset - raw data 64x256](https://github.com/eXascaleInfolab/ImputeGAP/raw/main/imputegap/dataset/docs/climate/01_climate-rawdata-NxM_graph.jpg)
![Climate dataset - raw data 20x400](https://github.com/eXascaleInfolab/ImputeGAP/raw/main/imputegap/dataset/docs/climate/02_climate-rawdata20x400_graph.jpg)
![Climate dataset - raw data 01x400](https://github.com/eXascaleInfolab/ImputeGAP/raw/main/imputegap/dataset/docs/climate/03_climate-rawdata01x400_graph.jpg)
![Climate - normalized 20x400](https://github.com/eXascaleInfolab/ImputeGAP/raw/main/imputegap/dataset/docs/climate/04_climate-normmin_maxdata01x400_graph.jpg)

### Features

| Category       | Feature                                                            | Value                 |
|----------------|--------------------------------------------------------------------|-----------------------|
| Geometry       | 5-bin histogram mode                                               | 0.09325838586722446   |
| Geometry       | 10-bin histogram mode                                              | -0.4947305240048063   |
| Geometry       | Proportion of high incremental changes in the series               | 0.8884977699553991    |
| Geometry       | Longest stretch of above-mean values                               | 119.0                 |
| Geometry       | Transition matrix column variance                                  | 0.0002728894456633256 |
| Geometry       | Goodness of exponential fit to embedding distance distribution     | 0.14380405574828758   |
| Geometry       | Positive outlier timing                                            | 0.06807999999999992   |
| Geometry       | Negative outlier timing                                            | -0.20139999999999997  |
| Geometry       | Longest stretch of decreasing values                               | 10.0                  |
| Geometry       | Rescaled range fluctuation analysis (low-scale scaling)            | 0.7                   |
| Geometry       | Detrended fluctuation analysis (low-scale scaling)                 | 0.64                  |
| Correlation    | First 1/e crossing of the ACF                                      | 3.3379314332379257    |
| Correlation    | First minimum of the ACF                                           | 6                     |
| Correlation    | Histogram-based automutual information (lag 2, 5 bins)             | 0.08356735782752395   |
| Correlation    | Time reversibility                                                 | -0.019876160881000596 |
| Correlation    | First minimum of the AMI function                                  | 5.0                   |
| Correlation    | Change in autocorrelation timescale after incremental differencing | 0.018867924528301886  |
| Trend          | Wangs periodicity metric                                           | 11                    |
| Trend          | Entropy of successive pairs in symbolized series                   | 1.8832855664772494    |
| Trend          | Error of 3-point rolling mean forecast                             | 0.7960326509462264    |
| Transformation | Power in the lowest 20% of frequencies                             | 0.7973876367136716    |
| Transformation | Centroid frequency                                                 | 0.4032451996154645    |


### Summary

| Data info          |                                                                         |
|--------------------|-------------------------------------------------------------------------|
| Dataset codename   | climate                                                                 |
| Dataset name       | Aggregated and Processed data collection for climate change attribution |
| Url                | https://viterbi-web.usc.edu/~liu32/data.html                            |
| Url item           | NA-1990-2002-Monthly.csv                                                |
| Time granularity   | 1 month                                                                 |
| Dataset dimensions | N=5000 M=10                                                             |

<br /><hr /><br />

<br /><hr /><br />

## Meteo

### Plots
The plots present a series of plots derived from the Meteo dataset, illustrating various aspects of the data and preprocessing steps.
Meteo dataset - raw data 64x256 shows the full raw dataset, consisting of NxM time series.
Meteo dataset - raw data 20x400 provides a subset of the data, limited to 20 time series over 400 time steps, while Meteo dataset - raw data 01x400 focuses on a single time series extracted from the dataset.
Finally, Meteo - normalized 20x400 demonstrates the impact of "MIN-MAX" normalization on the raw data, applied to the same 20x400 subset.

![Meteo dataset - raw data 64x256](https://github.com/eXascaleInfolab/ImputeGAP/raw/main/imputegap/dataset/docs/meteo/01_meteo-rawdata-NxM_graph.jpg)
![Meteo dataset - raw data 20x400](https://github.com/eXascaleInfolab/ImputeGAP/raw/main/imputegap/dataset/docs/meteo/02_meteo-rawdata20x400_graph.jpg)
![Meteo dataset - raw data 01x400](https://github.com/eXascaleInfolab/ImputeGAP/raw/main/imputegap/dataset/docs/meteo/03_meteo-rawdata01x400_graph.jpg)
![Meteo - normalized 20x400](https://github.com/eXascaleInfolab/ImputeGAP/raw/main/imputegap/dataset/docs/meteo/04_meteo-normmin_maxdata01x400_graph.jpg)

### Features

| Category       | Feature                                                            | Value                  |
|----------------|--------------------------------------------------------------------|------------------------|
| Geometry       | 5-bin histogram mode                                               | 0.8059092680204243     |
| Geometry       | 10-bin histogram mode                                              | -0.9508567159430887    |
| Geometry       | Proportion of high incremental changes in the series               | 0.5634428172140861     |
| Geometry       | Longest stretch of above-mean values                               | 4048.0                 |
| Geometry       | Transition matrix column variance                                  | 0.003505040957781977   |
| Geometry       | Goodness of exponential fit to embedding distance distribution     | 0.2534374512649519     |
| Geometry       | Positive outlier timing                                            | -0.142285              |
| Geometry       | Negative outlier timing                                            | 0.095275               |
| Geometry       | Longest stretch of decreasing values                               | 28.0                   |
| Geometry       | Rescaled range fluctuation analysis (low-scale scaling)            | 0.8                    |
| Geometry       | Detrended fluctuation analysis (low-scale scaling)                 | 0.7                    |
| Correlation    | First 1/e crossing of the ACF                                      | 754.468963708409       |
| Correlation    | First minimum of the ACF                                           | 13                     |
| Correlation    | Histogram-based automutual information (lag 2, 5 bins)             | 0.6302168700823286     |
| Correlation    | Time reversibility                                                 | 0.00026558821164784945 |
| Correlation    | First minimum of the AMI function                                  | 12.0                   |
| Correlation    | Change in autocorrelation timescale after incremental differencing | 0.0023030861354214646  |
| Trend          | Wangs periodicity metric                                           | 23                     |
| Trend          | Entropy of successive pairs in symbolized series                   | 1.4082420876782868     |
| Trend          | Error of 3-point rolling mean forecast                             | 0.3127146465659483     |
| Transformation | Power in the lowest 20% of frequencies                             | 0.9795268689123781     |
| Transformation | Centroid frequency                                                 | 0.0007669903939428711  |


### Summary

| Data info          |                                                                                                                              |
|--------------------|------------------------------------------------------------------------------------------------------------------------------|
| Dataset codename   | meteo                                                                                                                        |
| Dataset name       | Meteo Suisse data                                                                                                            |
| Dataset source     | Federal Office of Meteorology and Climatology, MeteoSwiss<br/>Operation Center 1<br/>Postfach 257<br/>8058 Zürich-Flughafen  | 
| Granularity        | 10 minutes                                                                                                                   |
| Dataset dimensions | N=10000 M=10                                                                                                                 |
|                    | TBA                                                                                                                          |

#### Dataset description

##### Stations
| stn       | Name       | Parameter  | Data source               | Longitude/Latitude | Coordinates [km] | Elevation [m]  |
|-----------|------------|------------|---------------------------|--------------------|------------------|----------------|
| ZHUST     | Aatal Höhe | erssurs0   | Kanton Zürich; Tiefbauamt | 8°45'/47°21'       | 698612/244769    | 490            |

##### Parameters

|            | Unit   | Description                      |
|------------|--------|----------------------------------|
| erssurs0   | mm     | null                             |
| merssurs0  | Code   | Mutation information on erssurs0 |

Observation interval for hourly values, unless otherwise indicated in the parameter description: HH  = (HH-1):41 - HH:40
Example: 13 = observation period 12:41 to 13:40

<br /><hr /><br />

## BAFU

### Plots
The plots present a series of plots derived from the BAFU dataset, illustrating various aspects of the data and preprocessing steps.
BAFU dataset - raw data 64x256 shows the full raw dataset, consisting of NxM time series.
BAFU dataset - raw data 20x400 provides a subset of the data, limited to 20 time series over 400 time steps, while BAFU dataset - raw data 01x400 focuses on a single time series extracted from the dataset.
Finally, BAFU - normalized 20x400 demonstrates the impact of "MIN-MAX" normalization on the raw data, applied to the same 20x400 subset.

![BAFU dataset - raw data 64x256](https://github.com/eXascaleInfolab/ImputeGAP/raw/main/imputegap/dataset/docs/bafu/01_bafu-rawdata-NxM_graph.jpg)
![BAFU dataset - raw data 20x400](https://github.com/eXascaleInfolab/ImputeGAP/raw/main/imputegap/dataset/docs/bafu/02_bafu-rawdata20x400_graph.jpg)
![BAFU dataset - raw data 01x400](https://github.com/eXascaleInfolab/ImputeGAP/raw/main/imputegap/dataset/docs/bafu/03_bafu-rawdata01x400_graph.jpg)
![BAFU - normalized 20x400](https://github.com/eXascaleInfolab/ImputeGAP/raw/main/imputegap/dataset/docs/bafu/04_bafu-normmin_maxdata01x400_graph.jpg)

### Features

| Category       | Feature                                                            | Value                 |
|----------------|--------------------------------------------------------------------|-----------------------|
| Geometry       | 5-bin histogram mode                                               | 10.677826122412835    |
| Geometry       | 10-bin histogram mode                                              | 4.917292125971148     |
| Geometry       | Proportion of high incremental changes in the series               | 0.004720104456518018  |
| Geometry       | Longest stretch of above-mean values                               | 170406.0              |
| Geometry       | Transition matrix column variance                                  | 0.00591715976331361   |
| Geometry       | Goodness of exponential fit to embedding distance distribution     | 0.014639882988124284  |
| Geometry       | Positive outlier timing                                            | -0.6863011474556843   |
| Geometry       | Negative outlier timing                                            | 0.5013873729015802    |
| Geometry       | Longest stretch of decreasing values                               | 213.0                 |
| Geometry       | Rescaled range fluctuation analysis (low-scale scaling)            | 0.2                   |
| Geometry       | Detrended fluctuation analysis (low-scale scaling)                 | 0.82                  |
| Correlation    | First 1/e crossing of the ACF                                      | 31.923359247023075    |
| Correlation    | First minimum of the ACF                                           | 60                    |
| Correlation    | Histogram-based automutual information (lag 2, 5 bins)             | 0.0006100829700701747 |
| Correlation    | Time reversibility                                                 | 1.265206120624004e-06 |
| Correlation    | First minimum of the AMI function                                  | 40.0                  |
| Correlation    | Change in autocorrelation timescale after incremental differencing | 2.542976299460889e-05 |
| Trend          | Wangs periodicity metric                                           | 3238                  |
| Trend          | Entropy of successive pairs in symbolized series                   | 1.1196301017972983    |
| Trend          | Error of 3-point rolling mean forecast                             | 0.27272853006237263   |
| Transformation | Power in the lowest 20% of frequencies                             | 0.9838423222377587    |
| Transformation | Centroid frequency                                                 | 0.03133275601505682   |


### Summary

| Data info          |                                             |
|--------------------|---------------------------------------------|
| Dataset codename   | BAFU<br/>bafu                                   |
| Dataset name       | Hydrological data across multiple stations  |
| Url/source         | https://www.bafu.admin.ch/bafu/en/home.html |
| Granularity        | 30 minutes                                  |
| Observations       | spans years 1974 to 2015                    |
| Dataset dimensions | N=50000 M=10                                |
